!     Last change:  WRG  13 Jul 2006    4:07 pm
MODULE mconcat
CONTAINS

   SUBROUTINE concat(i,f0,g0,aa)
   ! Calculates the i-th coefficient of the Taylor series expansion
   ! of a function composed of the taylor series expansion of
   ! two functions, f0 and g0.
   ! INPUTS :
   ! (1) i  : the order of the coefficient in the taylor series expansion
   ! (2) f0 : The coefficients in the expansion of the outer function
   ! (3) g  : The coefficients in the expansion of the inner function
   ! OUTPUT :
   ! (1) aa : i-th coefficient of the taylor series expansion
    IMPLICIT NONE
    INTEGER(4),INTENT(IN):: i
    REAL(8),INTENT(IN)::    f0(:),g0(:)
    REAL(8),INTENT(OUT)::   aa
    INTEGER(4)::            ll,j,d         ! generic loop counters
    REAL(8)::               qq(0:i,0:i)

     qq=0.d0
     qq(0,0)=1.d0

     IF (i==0) THEN
        aa = f0(1)
     ELSE
        DO d=1,i
           DO ll=1,d
              DO j=1,d+1-ll
                 qq(ll,d) = qq(ll,d) + g0(j+1)*qq(ll-1,d-j)
              END DO
           END DO
        END DO
        aa = DOT_PRODUCT(f0(2:i+1),qq(1:i,i))
     END IF

    RETURN
   END SUBROUTINE concat

END MODULE mconcat
