!     Last change:  WRG  18 Jul 2006    1:30 am
MODULE mrecursion
INTERFACE factorial
 MODULE PROCEDURE fact
END INTERFACE factorial


CONTAINS

SUBROUTINE recursion(tt,obj)
!********************************************************************************!
!                   Subroutine RECURSION                                         !
!********************************************************************************!
! This subroutine solves for the Bayesian Nash Equilibrium                       !
! of an asymetric first price auction model with n types.                        !
! The solution technique approximates the system of differential                 !
! equations by Taylor series expansion and solves backwards.                     !
! INPUT                                                                          !
! tt  : trial value of the bid corresponding to the highest value                !
! OUTPUT                                                                         !
! obj : the value of the objective function to be minimized                      !
!********************************************************************************!
 USE IMSL
 USE mpreamble
 USE mconcat
 USE cdf_mod
 IMPLICIT NONE
 REAL(8),INTENT(IN)::  tt
 REAL(8),INTENT(OUT):: obj
 INTEGER(4)::          i,m,j,h
 REAL(8)::             mxr(1),lj(n),fc,cdfres(n)
! REAL(8)            :: DPPDER

! Discretize t at nint interval between [0,t*]

  t(nt)=tt
  inc = (t(nt)-res)/dble(nt)
  DO i=nt-1,0,-1
     t(i) = t(i+1) - inc
  END DO

  t(0)=res

  cdfres=f_cdf(res)

! initialize l and lpl
  l=0.0d0
  p0=0.0d0
  DO i=1,n
     bids(i,:)=t
  END DO
  lpl=0.0d0
  lpl(:,nt)=f_pdf(uv)*DBLE(bigN/(bigN-1))
  l(:,nt)=1.0d0
  check=0.0d0
  obj=0.0d0

! *** Recursion over the grid of t from t=t(nt) to t(1) ***

  m=nt
  DO WHILE(m>=1)

! initialize a
     a=0.0d0
     a(:,0)=l(:,m)

! taylor expansion of the inverse distribution function

     DO i=1,n
        DO j=0,bigJ
           fc=DBLE(fact(j))
           d(i,j) = ((-1.0d0)**j)*DPPDER(j,1.0d0-a(i,0),ko,npc,br(i,:),pcf(i,:,:))/fc
        END DO
     END DO
     
! initialize other taylor series coefficients
     p(:,0) = d(:,0) - t(m)
     bigb(:,1)=-1.0d0
     a1=0.0d0
     DO i=1,n
        a1(i,i) = 1.0d0/p(i,0)
        a2(:,i) = k(i)/p(i,0)
     END DO
     biga = a1-a2*sumk

     b2(:,:)=MATMUL(biga(:,:),bigb(:,:))
     b(:,0)=b2(:,1)

! need to store p0 for revenue calculations

     p0(:,m)=p(:,0)

! recursion to calculate a(:,i),i=1,...,bigJ

     DO i=1,bigJ

! calculate a(:,i)
        DO j=0,i-1        
           a(:,i) = a(:,i)+a(:,j)*b(:,i-j-1)
        END DO
        a(:,i) = a(:,i)/dble(i)

! calculate p(:,i)
        DO j=1,n
           call concat(i,d(j,0:bigJ),a(j,0:bigJ),p(j,i))
        END DO
         
        IF (i==1) THEN
           p(:,i)=p(:,i)-1.0d0
        END IF

! calculate RHS of main equation
        DO j=1,i
           c1(j,:) = b(:,i-j)
        END DO
  
        c2 = MATMUL(p(:,1:i),c1(1:i,:))

        DO j=1,n
           bigb(j,1)=sum(b1(j,:)*c2(j,:))
        END DO

! calculate new b
        b2(:,:) = MATMUL(biga(:,:),bigb(:,:))
        b(:,i) = b2(:,1)        

     END DO   ! i

  m=m-1  

! calculate new values of l and inverse bids, and lpl
  DO i=0,bigJ
     l(:,m) = l(:,m) + a(:,i)*((-inc)**i)
     lpl(:,m)=lpl(:,m)+b(:,i)*((-inc)**i)
     bids(:,m+1)=bids(:,m+1)+p(:,i)*((-inc)**i)
  END DO

! Calculation of the objective function

  WHERE ((l(:,m)-cdfres<0.0d0) .OR. (l(:,m)>1.0d0) &
		& .OR. (l(:,m)>l(:,m+1)))
     check=1.0d0
  ELSEWHERE
     check=0.0d0
  END WHERE

  IF (sum(check)>0.0d0) THEN

! Penalty if l(t) < 0 or if derivitive gets too large
     obj = 1000.0d0
     m = 0

! Calculation of objective function

  ELSE IF ((sum(check)==0.0d0) .AND. (m<=2)) THEN
      obj = obj+SQRT(SUM(p(:,0)**2))
  END IF 

  END DO ! m

 RETURN
END SUBROUTINE recursion

INTEGER(4) FUNCTION fact(n)
  IMPLICIT NONE
  INTEGER(4),INTENT(IN):: n
  INTEGER(4)::            i,temp

  IF (n<=0) THEN
     fact=1
  ELSE
     temp=1
     DO i=1,n
        temp=temp*i
     END DO
     fact=temp
  END IF

  RETURN
END FUNCTION fact


END MODULE mrecursion

