!     Last change:  WRG  21 Jul 2006    5:31 pm
MODULE cdf_mod
!================================================================================!
! This module contains the subroutine that asks the user for the type of         !
! distribution to be used.  It currently contains the Uniform and Weibull as     !
! options.  To add another distribution, add another case replicate one of the   !
! first two alternatives.                                                        !
! This module also contains functions that returns the vector of values of the   !
! pdf and cdf for each player.  To add another distribution function, these      !
! functions have to be extended as well.                                         !
!================================================================================!
USE mpreamble
IMPLICIT NONE
REAL(8)                  :: ht

CONTAINS
!--------------------------------------------------------------------------------!
SUBROUTINE getcdf
USE mpreamble
IMPLICIT NONE
INTEGER(4)         :: i,j

WRITE(*,200)
200 FORMAT (1X,'Here is a menu of b-CDFs to choose from (see user guide for parametrizations): ',/, &
            1x,'1 - two parameter Weibull;',/, &
            1x,'2 - Beta;',/, &
            1x,'3 - normal;',/, &
            1x,'4 - lognormal.')
WRITE(*,*) ' '
WRITE(*,'(1x,"(9) Enter the number of b-CDFs: ")')
READ(*,*) ncdf

IF (ALLOCATED(ccl)) DEALLOCATE(ccl,ccu,icdf,musd,qcdf)
ALLOCATE(ccl(ncdf),ccu(ncdf),icdf(ncdf),musd(ncdf,2),qcdf(n,ncdf))

WRITE(*,210)
210 FORMAT (1x,'(10) Enter the index of each b-CDF: ')
READ(*,*) icdf(:)

DO j=1,ncdf
  IF (j==1) THEN
  SELECT CASE (icdf(j))
      CASE (1)
         WRITE (*,220)
         220 FORMAT (1x,'(11) Enter alpha and beta: ')
         READ(*,*) musd(j,:)
      CASE (2)
         WRITE (*,230)
         230 FORMAT (1x,'(11) Enter alpha and beta: ')
         READ(*,*) musd(j,:)
      CASE (3)
         WRITE (*,240)
         240 FORMAT (1x,'(11) Enter mu and sigma: ')
         READ(*,*) musd(j,:)
      CASE (4)
         WRITE (*,250)
         250 FORMAT (1x,'(11) Enter mu and sigma: ')
         READ(*,*) musd(j,:)
      CASE DEFAULT
         WRITE(*,*) 'Error in choice of distribution: '
         STOP
  END SELECT
  ELSE
  SELECT CASE (icdf(j))
      CASE (1)
         WRITE (*,225)
         225 FORMAT (1x,'     Enter alpha and beta: ')
         READ(*,*) musd(j,:)
      CASE (2)
         WRITE (*,235)
         235 FORMAT (1x,'     Enter alpha and beta: ')
         READ(*,*) musd(j,:)
      CASE (3)
         WRITE (*,245)
         245 FORMAT (1x,'     Enter mu and sigma: ')
         READ(*,*) musd(j,:)
      CASE (4)
         WRITE (*,255)
         255 FORMAT (1x,'     Enter mu and sigma: ')
         READ(*,*) musd(j,:)
      CASE DEFAULT
         WRITE(*,*) 'Error in choice of distribution: '
         STOP
  END SELECT
  END IF
  aa = musd(j,1)
  bb = musd(j,2)
  ccl(j) = ccdf0(lv,icdf(j))
  ccu(j) = ccdf0(uv,icdf(j))
END DO

DO i=1,n
   IF (i==1) THEN
      WRITE(*,270) i
      270 FORMAT (1x,"(12) Enter the exponents of the b-CDFs for h-type",I2,": ")
   ELSE
      WRITE(*,275) i
      275 FORMAT (1x,"     Enter the exponents of the b-CDFs for h-type",I2,": ")
   END IF
   READ(*,*) qcdf(i,:)
!  WRITE(*,'(1x,"(12) - For type ",I6)') i
!  WRITE(*,270)
!  270 FORMAT (1x,'Enter the sequence of exponents corresponding',/, &
!              1x,'to the use of the cdfs: ')
END DO

END SUBROUTINE getcdf

!--------------------------------------------------------------------------------!
FUNCTION ccdf(x,i)
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++!
! Contains the user provided distributions of private values for each type of    !
! bidder.  This is the only routine that required modification by the user.      !
! Simply follow the instructions to specify the distribution functions           !
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++!
USE mpreamble
IMPLICIT NONE
INTEGER(4),INTENT(IN)  :: i
REAL(8),INTENT(IN)     :: x
REAL(8)                :: ccdf
REAL(8)                :: rcdf
INTEGER(4)             :: j

IF (aucpro==1) THEN
 rcdf=1.0d0
 DO j=1,ncdf
    aa = musd(j,1)
    bb = musd(j,2)
    rcdf=rcdf*((ccdf0(x,icdf(j))-ccl(j))/(ccu(j)-ccl(j)))**qcdf(i,j)
 END DO

 ccdf=rcdf

ELSE ! change date 11/24/10.  Makes the cdf correct for collusion in procurements.

 rcdf=1.0d0
 DO j=1,ncdf
    aa = musd(j,1)
    bb = musd(j,2)
    rcdf=rcdf*((ccu(j) - ccdf0(x,icdf(j)))/(ccu(j)-ccl(j)))**qcdf(i,j)
 END DO

 ccdf=1.0d0 - rcdf

END IF

 RETURN
END FUNCTION ccdf
!--------------------------------------------------------------------------------!
FUNCTION ccdf0(x,i)
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++!
! Contains the user provided distributions of private values for each type of    !
! bidder.  This is the only routine that required modification by the user.      !
! Simply follow the instructions to specify the distribution functions           !
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++!
USE IMSL
USE mpreamble
IMPLICIT NONE
INTEGER(4),INTENT(IN)  :: i
REAL(8),INTENT(IN)     :: x
REAL(8)                :: ccdf0,pb
!REAL(8),EXTERNAL       :: DBETDF,DNORDF
! Here you are required to enter the distribution function for
! each type in each case.  Simply remove comments from the required number
! of cases and enter the distribution function as in case one.  Note user should
! also change the distribution for case 1 to the distribution required for
! the first type.  Finally, the number of cases must be the same as what you
! intered for n.

 pb=0.0d0
 cdftype: SELECT CASE (i)
     CASE (1)
      ccdf0 = 1.0d0 - EXP(-(x/aa)**bb)
     CASE (2)
      ccdf0 = DBETDF(x,aa,bb)
!      ccdf0 = pb*x+(1.0d0-pb)*DBETDF(x,aa,bb)
     CASE (3)
      ccdf0 = DNORDF((x-aa)/bb)
     CASE (4)
      ccdf0 = DNORDF((log(x)-aa)/bb)
!     CASE (5)
!      ccdf0 = DBETDF(x,aa,bb)
!     CASE (6)
!      ccdf0 =
!     CASE (7)
!      ccdf0 =
     CASE DEFAULT
      WRITE(*,*) 'Error in choice of distribution'
         STOP
 END SELECT cdftype

 RETURN
END FUNCTION ccdf0

!_______________________________________________________________________________!

SUBROUTINE askcdf
USE IMSL
USE mpreamble
IMPLICIT NONE
REAL(8)               :: h(npt),ant,dh,v1(n,npt),au,bu,xk(nko), &
                         erb,era
REAL(8)               :: h1(npt),ant1,dh1,v2(n,npt),au1,bu1,xk1(nko)
INTEGER(4)            :: i,j,maxfn

! construct equal spaced grid for the inverse cdf
 ant=DBLE(npt-1)
 dh=1.d0/ant
 DO j=1,npt
    h(j) = -dh+DBLE(j)*dh
 END DO

 ! construct equal spaced grid for the cdf
 ant1=DBLE(npt-1)
 dh1=(uv-lv)/ant1
 DO j=1,npt
    h1(j) = lv-dh1+DBLE(j)*dh1
 END DO

! allocate memory to spline stuff
 IF (ALLOCATED(bcf)) DEALLOCATE(bcf,pcf,br,bcf1,pcf1,br1)
 ALLOCATE(bcf(n,npt),pcf(n,ko,npt),br(n,npt),bcf1(n,npt),pcf1(n,ko,npt),br1(n,npt))

! construct corresponding (unequal spaced) grid for cdf for each type
! in the same loop, construct the spline breakpoints (using dbsnak), the
! spline coefficients (using bsint), and the polynomial approximations
! (using bsccp).
 CALL dbsnak(npt,h,ko,xk)    ! get spline knots. Store in xk
 era=0.0d0
 erb=1.0d-12
 v1(:,1)=uv
 v1(:,npt)=lv
 DO i=1,n
    dtype=i
    DO j=2,npt-1
       maxfn=100
       ht=h(j)
       au=lv
       bu=v1(i,j-1)
       CALL dzbren(hzr,era,erb,au,bu,maxfn)
       v1(i,j)=bu
    END DO
    CALL dbsint(npt,h,v1(i,:),ko,xk,bcf(i,:))  ! get spline coefficients
    CALL dbscpp(ko,xk,npt,bcf(i,:),npc,br(i,:),pcf(i,:,:)) ! convert to picewise polynomials
 END DO


! do same for the cdf
 CALL dbsnak(npt,h1,ko,xk1)
 v2(:,1)=1.d0
 v2(:,npt)=0.d0
 DO i=1,n
    dtype=i
    DO j=2,npt-1
       v2(i,j)=hdf(h1(j))
    END DO
    CALL dbsint(npt,h1,v2(i,:),ko,xk1,bcf1(i,:))
    CALL dbscpp(ko,xk1,npt,bcf1(i,:),npc1,br1(i,:),pcf1(i,:,:))
 END DO

 Fres = f_cdf(res)

END SUBROUTINE askcdf

!________________________________________________________________________________!

FUNCTION hdf(vv)
USE mpreamble
IMPLICIT NONE
REAL(8),INTENT(IN)    :: vv
REAL(8)               :: hdf
 hdf =1.0d0 - ccdf(vv,dtype)
END FUNCTION hdf

!________________________________________________________________________________!

FUNCTION hzr(vv)
USE mpreamble
IMPLICIT NONE
REAL(8),INTENT(IN)    :: vv
REAL(8)               :: hzr
 hzr = ht-hdf(vv)
END FUNCTION hzr

!________________________________________________________________________________!

FUNCTION f_cdf(vv)
USE IMSL
USE mpreamble
IMPLICIT NONE
REAL(8),INTENT(IN)   :: vv
REAL(8)              :: f_cdf(n)
INTEGER(4)           :: i,j
!REAL(8),EXTERNAL     :: DPPDER
 j=0
 DO i=1,n
   dtype=i
   f_cdf(i) = 1.0d0-DPPDER(j,vv,ko,npc1,br1(i,:),pcf1(i,:,:))
 END DO
END FUNCTION f_cdf

!________________________________________________________________________________!

FUNCTION f_pdf(vv)
USE mpreamble
USE IMSL
IMPLICIT NONE
REAL(8),INTENT(IN)   :: vv
REAL(8)              :: f_pdf(n)
INTEGER(4)           :: i,j
!REAL(8),EXTERNAL     :: DPPDER
 j=1
 DO i=1,n
    f_pdf(i) = -DPPDER(j,vv,ko,npc1,br1(i,:),pcf1(i,:,:))
 END DO
END FUNCTION f_pdf

!________________________________________________________________________________!

SUBROUTINE stats
!***********************************************************************************!
!                          Subroutine STATS                                         !
!-----------------------------------------------------------------------------------!
! Computes the mean and standard deviation of private values distributions          !
!***********************************************************************************!
USE mpreamble
USE quadrature
IMPLICIT NONE
!INTEGER(4),PARAMETER:: nt=5000
REAL(8)           :: qa(0:nt),qw(0:nt),d1(0:nt,n),d2(0:nt,n),cunc(0:nt,n),punc(0:nt,n),ctemp(n),ptemp(n),inn
REAL(8)           :: cunc1(0:nt,n),punc1(0:nt,n)
INTEGER(4)        :: i,j,citemp(n),pitemp(n)

 CALL gauleg(lv,uv,qa,qw)
 DO j=0,nt
    cunc(j,:)=f_cdf(qa(j))
	punc(j,:)=f_pdf(qa(j))
    d1(j,:)=cunc(j,:)*qw(j)
    d2(j,:)=qa(j)*d1(j,:)
 END DO

 DO j=1,n
	citemp(j:j) = minloc((1d-6-cunc(1:nt,j))**2)
	pitemp(j:j) = minloc((1d-6-punc(1:nt,j))**2)
	cf(j) = qa(citemp(j))
!	cf1(j)= qa(citemp(j)+1)
	pf(j) = qa(pitemp(j))
!	pf1(j)= qa(pitemp(j)+1)
	ctemp = f_cdf(cf(j))
	ptemp = f_pdf(pf(j))
    cff(j)= ctemp(j)
	pff(j)= ptemp(j)
!	ctemp = f_cdf(cf1(j))
!	ptemp = f_pdf(pf1(j))
!	cff1(j)=ctemp(j)
!	pff1(j)=ptemp(j)
 END DO

! print *, pf
! print *, pff
! print *, cf
! print *, cff
! print *, pf1
! print *, pff1
! print *, cf1
! print *, cff1

 tmean = uv-SUM(d1,DIM=1)
 tstd = SQRT(uv**2 - 2.0d0*SUM(d2,DIM=1) - tmean**2)


 inn = (uv-lv)/dble(nt)

 uf = f_pdf(uv-inn)
 lf = f_pdf(lv+inn)

 uff= 1.0d0-f_cdf(uv-inn)
 lff= f_cdf(lv+inn)

END SUBROUTINE stats


END MODULE cdf_mod

